
#include "C64.h"
#include "main.h"
#include "prefs.h"
#include <time.h>

#define SAMPLERATE 44100
#define SAMPLESPERBUF (SAMPLERATE / 50)
#define BYTESPERSAMPLE 4

int16* sound_calc_buf;
int sndPlaying=0;
int sample_length = SAMPLE_FREQ;

u32 *audioBuffer;
ndspWaveBuf waveBuf[2];
bool fillBlock = false;
size_t stream_offset = 0;

bool reInitSound = 0;

void DigitalRenderer::init_sound(void)
{
    if(useCSND){
        sprintf(debugText, "Init Sound!");
        csndInit();//start Audio Lib
        if(sndPlaying==0){
            sound_calc_buf = (int16*)linearAlloc( sample_length*4); // give it loads of space for experimenting
            csndPlaySound(8,SOUND_FORMAT_16BIT | SOUND_REPEAT, 44100, 1, 0, (u32*)sound_calc_buf, (u32*)sound_calc_buf, sample_length);
            ready = true;
            sndPlaying=1;
        }
    }else{
        //DSP here
        audioBuffer = (u32*)linearAlloc(SAMPLESPERBUF*BYTESPERSAMPLE*2);
        ndspInit();
        ndspSetOutputMode(NDSP_OUTPUT_MONO);
        ndspChnSetInterp(0, NDSP_INTERP_LINEAR);
        ndspChnSetRate(0, SAMPLERATE);
        ndspChnSetFormat(0, NDSP_FORMAT_MONO_PCM16);
        memset(waveBuf,0,sizeof(waveBuf));
        waveBuf[0].data_vaddr = &audioBuffer[0];
        waveBuf[0].nsamples = SAMPLESPERBUF;
        waveBuf[1].data_vaddr = &audioBuffer[SAMPLESPERBUF];
        waveBuf[1].nsamples = SAMPLESPERBUF;
        stream_offset = 0;
        ndspChnWaveBufAdd(0, &waveBuf[0]);
        ndspChnWaveBufAdd(0, &waveBuf[1]);
        ready = true;
        sndPlaying=1;
    }

}




DigitalRenderer::~DigitalRenderer()
{
    if(useCSND){
        CSND_SetPlayState(8, 0);
        csndExecCmds(0);
        memset(sound_calc_buf, 0, sample_length);
        linearFree(sound_calc_buf);
    }else{
        if(sndPlaying){
            ndspExit();
            //linearFree(audioBuffer);
        }
    }
}



int thingyCount = 50;//CALC_FREQ*5;//160;// SAMPLE_FREQ / (CALC_FREQ * 4);
u64 thisTime = 0;
u64 oldTime = 0;
u64 framecnt=0;

u64 TPS = 268123480;//268111856LL;//268123324;
int offset = sample_length;
void DigitalRenderer::EmulateLine(void)
{
    
    if(reInitSound==1){
        init_sound();
        reInitSound=0;
        sndPlaying=1;
    }
    
    sample_buf[sample_in_ptr] = volume;// << 1 ; // double the volume!
	sample_in_ptr = (sample_in_ptr + 1) % SAMPLE_BUF_SIZE;

    if(useCSND){
        if(sndPlaying!=0){
            int divide = 50;
            thisTime = svcGetSystemTick();
            if (oldTime < thisTime){
                oldTime = thisTime + (TPS/CALC_FREQ) ;//  50ish times per second
                calc_buffer(sound_calc_buf + offset, sample_length/(divide/2)); // 2 50ths of a second of sound
                if(showMenu == 1 || filesGot == 1){
                    memset(sound_calc_buf, 0, sample_length);
                }
                offset += (sample_length/divide); // increment by 50th of sample length
                if(offset >= sample_length/2) offset -= (sample_length/2);
            }
        }
    }else{
    
        if(sndPlaying!=0){
            if (waveBuf[fillBlock].status == NDSP_WBUF_DONE) {

                calc_buffer(waveBuf[fillBlock].data_pcm16,  SAMPLESPERBUF * 2);
                
                DSP_FlushDataCache(waveBuf[fillBlock].data_pcm16, SAMPLESPERBUF * 2);
                
                ndspChnWaveBufAdd(0, &waveBuf[fillBlock]);
                stream_offset += waveBuf[fillBlock].nsamples;

                fillBlock = !fillBlock;
            }
        }
    }
    return;
}




void DigitalRenderer::Pause(void)
{
  if (ready){
    if(useCSND){
        CSND_SetPlayState(8, 0);
        csndExecCmds(0);
        memset(sound_calc_buf, 0, sample_length);
    }else{
        if(sndPlaying){
            sndPlaying=0;
            ndspChnWaveBufClear(0);
            ndspExit();
            linearFree(audioBuffer);
        }
    }
  }
}




void DigitalRenderer::Resume(void)
{
  if (ready){
    if(useCSND){
        offset=0;
        csndPlaySound(8,SOUND_FORMAT_16BIT | SOUND_REPEAT, SAMPLE_FREQ, 1, 0, (u32*)sound_calc_buf, (u32*)sound_calc_buf, sample_length);
    }else{
        reInitSound=1;
    }
  }
}
